@@ -4,7 +4,19 @@ from typing import Union
 
 from ._cffi_ppmd import ffi, lib
 
-__all__ = ("Ppmd7Encoder", "Ppmd7Decoder", "Ppmd8Encoder", "Ppmd8Decoder", "PpmdError")
+__all__ = (
+    "Ppmd7Encoder",
+    "Ppmd7Decoder",
+    "Ppmd8Encoder",
+    "Ppmd8Decoder",
+    "PpmdError",
+    "PPMD8_RESTORE_METHOD_RESTART",
+    "PPMD8_RESTORE_METHOD_CUT_OFF",
+)
+
+PPMD8_RESTORE_METHOD_RESTART = 0
+PPMD8_RESTORE_METHOD_CUT_OFF = 1
+# PPMD8_RESTORE_METHOD_FREEZE = 2
 
 _PPMD7_MIN_ORDER = 2
 _PPMD7_MAX_ORDER = 64
@@ -179,7 +191,7 @@ class PpmdBaseEncoder:
 
     def _setup_inBuffer(self, data):
         # Input buffer
-        in_buf = _new_nonzero("PPMD_inBuffer *")
+        in_buf = _new_nonzero("InBuffer *")
         if in_buf == ffi.NULL:
             raise MemoryError
         in_buf.src = ffi.from_buffer(data)
@@ -189,7 +201,7 @@ class PpmdBaseEncoder:
 
     def _setup_outBuffer(self):
         # Output buffer
-        out_buf = _new_nonzero("PPMD_outBuffer *")
+        out_buf = _new_nonzero("OutBuffer *")
         self.writer.outBuffer = out_buf
         if out_buf == ffi.NULL:
             raise MemoryError
@@ -240,7 +252,7 @@ class PpmdBaseDecoder:
 
     def _setup_inBuffer(self, data):
         # Input buffer
-        in_buf = _new_nonzero("PPMD_inBuffer *")
+        in_buf = _new_nonzero("InBuffer *")
         # Prepare input buffer w/wo unconsumed data
         if self._in_begin == self._in_end:
             # No unconsumed data
@@ -311,7 +323,7 @@ class PpmdBaseDecoder:
 
     def _setup_outBuffer(self):
         # Output buffer
-        out_buf = _new_nonzero("PPMD_outBuffer *")
+        out_buf = _new_nonzero("OutBuffer *")
         if out_buf == ffi.NULL:
             raise MemoryError
         out = _BlocksOutputBuffer()
@@ -478,18 +490,18 @@ class Ppmd7Decoder(PpmdBaseDecoder):
 
 
 class Ppmd8Encoder(PpmdBaseEncoder):
-    def __init__(self, max_order, mem_size, end_mark=False):
+    def __init__(self, max_order, mem_size, restore_method=PPMD8_RESTORE_METHOD_RESTART, endmark=True):
         self.lock = Lock()
         if mem_size > sys.maxsize:
             raise ValueError("Mem_size exceed to platform limit.")
         self._init_common()
         self.ppmd = ffi.new("CPpmd8 *")
-        self.endmark = end_mark
+        self.endmark = endmark
         lib.ppmd8_compress_init(self.ppmd, self.writer)
         lib.Ppmd8_Construct(self.ppmd)
         lib.Ppmd8_Alloc(self.ppmd, mem_size, self._allocator)
         lib.Ppmd8_RangeEnc_Init(self.ppmd)
-        lib.Ppmd8_Init(self.ppmd, max_order, 0)
+        lib.Ppmd8_Init(self.ppmd, max_order, restore_method)
 
     def encode(self, data) -> bytes:
         self.lock.acquire()
@@ -529,110 +541,81 @@ class Ppmd8Encoder(PpmdBaseEncoder):
 
 
 class Ppmd8Decoder(PpmdBaseDecoder):
-    def __init__(self, max_order: int, mem_size: int, end_mark=False):
+    def __init__(self, max_order: int, mem_size: int, restore_method=PPMD8_RESTORE_METHOD_RESTART, endmark=True):
         self._init_common()
         self.ppmd = ffi.new("CPpmd8 *")
-        self.endmark = end_mark
+        self.args = ffi.new("ppmd8_args *")
+        self.args.endmark = endmark
         lib.Ppmd8_Construct(self.ppmd)
         lib.ppmd8_decompress_init(self.ppmd, self.reader)
         lib.Ppmd8_Alloc(self.ppmd, mem_size, self._allocator)
-        lib.Ppmd8_Init(self.ppmd, max_order, 0)
-        self.inited = False
-        self.flushed = False
+        lib.Ppmd8_Init(self.ppmd, max_order, restore_method)
+        self._inited = False
+        self._eof = False
+        self._needs_input = True
+        self._finished = False
 
     def _init2(self):
         lib.Ppmd8_RangeDec_Init(self.ppmd)
+        self.args.finished = True
 
     def decode(self, data: Union[bytes, bytearray, memoryview], length: int = -1):
-        if not isinstance(length, int) or (not self.endmark and length < 0):
-            raise PpmdError("Wrong length argument is specified. It should be positive integer without endmark mode.")
+        if not isinstance(length, int):
+            raise PpmdError("Wrong length argument is specified.")
         self.lock.acquire()
         in_buf, use_input_buffer = self._setup_inBuffer(data)
         out, out_buf = self._setup_outBuffer()
-        if not self.inited:
-            self.inited = True
+        if not self._inited:
+            self._inited = True
             self._init2()
-        if self.endmark:
-            while True:
-                if out_buf.pos == out_buf.size:
-                    out.grow(out_buf)
-                size = lib.ppmd8_decompress(self.ppmd, out_buf, in_buf, -1)
-                if size == -1:
-                    self.flushed = True
-                    res = out.finish(out_buf)
-                    self.lock.release()
-                    lib.Ppmd8_Free(self.ppmd, self._allocator)
-                    ffi.release(self.ppmd)
-                    self._release()
-                    return res
-                elif size == -2:
-                    raise ValueError("Corrupted archive data.")
-                if in_buf.pos == in_buf.size:
-                    break
-        else:
-            remaining: int = length
-            while remaining > 0:
-                if out_buf.pos == out_buf.size:
-                    out.grow(out_buf)
-                size = lib.ppmd8_decompress(self.ppmd, out_buf, in_buf, min(out_buf.size, remaining))
-                if size <= 0:
-                    break
-                remaining = remaining - size
+        if length < 0:
+            length = 0x7FFFFFFF
+        while True:
+            if out_buf.pos == length:
+                break
+            if out_buf.pos == out_buf.size:
+                out.grow(out_buf)
+            size = lib.ppmd8_decompress(self.ppmd, out_buf, in_buf, length, self.args)
+            if size == -1:
+                self._eof = True
+                self._needs_input = False
+                res = out.finish(out_buf)
+                self.lock.release()
+                self._free()
+                return res
+            elif size == -2:
+                raise ValueError("Corrupted archive data.")
+            if in_buf.pos == in_buf.size:
+                break
         self._unconsumed_in(in_buf, use_input_buffer)
+        if self._eof:
+            self._needs_input = False
+        elif self._input_buffer_size == 0 or (self._in_begin == self._in_end):
+            self._needs_input = True
+        else:
+            self._needs_input = False
         res = out.finish(out_buf)
         self.lock.release()
         return res
 
-    def flush(self, length: int = -1):
-        if self.flushed:
-            return b""
-        if not isinstance(length, int) or (not self.endmark and length < 0):
-            raise PpmdError("Wrong length argument is specified. It should be positive integer.")
-        self.lock.acquire()
-        if self.endmark:
-            if length == -1:
-                in_buf, use_input_buffer = self._setup_inBuffer(b"")
-                out, out_buf = self._setup_outBuffer()
-                while True:
-                    if out_buf.pos == out_buf.size:
-                        out.grow(out_buf)
-                    size = lib.ppmd8_decompress_flush(self.ppmd, out_buf, in_buf, -1)
-                    if size == -1:
-                        break
-                    elif size == -2:
-                        raise ValueError("Corrupted archive")
-                    elif size == 0:
-                        break
-                    else:
-                        pass
-                self._unconsumed_in(in_buf, use_input_buffer)
-                res = out.finish(out_buf)
-            else:
-                # FIXME
-                res = b""
-        elif length > 0:
-            in_buf, use_input_buffer = self._setup_inBuffer(b"")
-            out, out_buf = self._setup_outBuffer()
-            remaining: int = length
-            while remaining > 0:
-                if out_buf.pos == out_buf.size:
-                    out.grow(out_buf)
-                size = lib.ppmd8_decompress_flush(self.ppmd, out_buf, in_buf, min(out_buf.size - out_buf.pos, remaining))
-                remaining = remaining - size
-            self._unconsumed_in(in_buf, use_input_buffer)
-            res = out.finish(out_buf)
-        else:
-            res = b""
-        lib.Ppmd8_Free(self.ppmd, self._allocator)
+    def _free(self):
+        if self._finished:
+            return
+        self._finished = True
+        lib.Ppmd8T_Free(self.ppmd, self.args, self._allocator)
         ffi.release(self.ppmd)
         self._release()
-        self.flushed = True
-        self.lock.release()
-        return res
+
+    @property
+    def needs_input(self):
+        return self._needs_input
+
+    @property
+    def eof(self):
+        return self._eof
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        if not self.flushed:
-            self.flush(0)
+        self._free()
